@@ -3,9 +3,8 @@ import logging
 import os
 from typing import List, Optional, Tuple, Union
 
-import boto3
 import einops
-import gcsfs
+import numpy as np
 import torch
 from nowcasting_dataset.config.model import Configuration
 from nowcasting_dataset.consts import (
@@ -17,12 +16,11 @@ from nowcasting_dataset.consts import (
     PV_SYSTEM_ID,
     PV_YIELD,
     SATELLITE_DATA,
-    SPATIAL_AND_TEMPORAL_LOCATIONS_OF_EACH_EXAMPLE_FILENAME,
     TOPOGRAPHIC_DATA,
 )
-from nowcasting_dataset.dataset.batch import Batch, Example
-from nowcasting_dataset.filesystem.utils import delete_all_files_in_temp_path, download_to_local
-from nowcasting_dataset.utils import get_netcdf_filename, set_fsspec_for_multiprocess
+from nowcasting_dataset.dataset.batch import Batch, Example, join_two_batches
+from nowcasting_dataset.filesystem.utils import delete_all_files_in_temp_path
+from nowcasting_dataset.utils import set_fsspec_for_multiprocess
 
 from nowcasting_dataloader.batch import BatchML
 from nowcasting_dataloader.subset import subselect_data
@@ -44,7 +42,6 @@ class NetCDFDataset(torch.utils.data.Dataset):
         src_path: str,
         tmp_path: str,
         configuration: Configuration,
-        cloud: str = "gcp",
         required_keys: Union[Tuple[str], List[str]] = None,
         history_minutes: Optional[int] = None,
         forecast_minutes: Optional[int] = None,
@@ -52,6 +49,8 @@ class NetCDFDataset(torch.utils.data.Dataset):
         add_position_encoding: bool = False,
         data_sources_names: Optional[list[str]] = None,
         num_bands: int = 4,
+        mix_two_batches: bool = True,
+        seed: bool = 234,
     ):
         """
         Netcdf Dataset
@@ -62,29 +61,29 @@ class NetCDFDataset(torch.utils.data.Dataset):
                 Google Cloud storage.
             tmp_path: The full path to the local temporary directory
                 (on a local filesystem).
-            cloud:
             required_keys: Tuple or list of keys required in the example for
                 it to be considered usable
             history_minutes: How many past minutes of data to use, if subsetting the batch
             forecast_minutes: How many future minutes of data to use, if reducing the amount
                 of forecast time
             configuration: configuration object
-            cloud: which cloud is used, can be "gcp", "aws" or "local".
             normalize: normalize the batch data
             add_position_encoding: Whether to add position encoding or not
             data_sources_names: Names of data sources to load, if not using all of them
             num_bands: Number of bands for the Fourier features for the position encoding
-
+            mix_two_batches: option to mix tow batches together
+            seed: random seed for peaking second batch when mixing two batches
         """
         self.n_batches = n_batches
         self.src_path = src_path
         self.tmp_path = tmp_path
-        self.cloud = cloud
         self.history_minutes = history_minutes
         self.forecast_minutes = forecast_minutes
         self.configuration = configuration
         self.normalize = normalize
         self.add_position_encoding = add_position_encoding
+        self.seed = seed
+        self.mix_two_batches = mix_two_batches
 
         self.num_bands = num_bands
         if data_sources_names is None:
@@ -115,24 +114,23 @@ class NetCDFDataset(torch.utils.data.Dataset):
             required_keys = DEFAULT_REQUIRED_KEYS
         self.required_keys = list(required_keys)
 
-        # setup cloud connections as None
-        self.gcs = None
-        self.s3_resource = None
-
-        assert cloud in ["gcp", "aws", "local"]
-
         if not os.path.isdir(self.tmp_path):
             os.mkdir(self.tmp_path)
 
+        # set seed
+        np.random.seed(seed)
+
+        if len(self) == 1:
+            logger.warning("Wanted to mix batches but there is only one")
+            self.mix_two_batches = False
+
     def per_worker_init(self, worker_id: int):
         """Function called by a worker"""
-        if self.cloud == "gcp":
-            self.gcs = gcsfs.GCSFileSystem()
-        elif self.cloud == "aws":
-            self.s3_resource = boto3.resource("s3")
 
         # adjust temp path for each worker
-        self.tmp_path = f"{self.tmp_path}/{worker_id}"
+        if self.src_path != self.tmp_path:
+            self.tmp_path = f"{self.tmp_path}/{worker_id}"
+            os.mkdir(f"{self.tmp_path}")
 
     def __len__(self):
         """Length of dataset"""
@@ -155,30 +153,39 @@ class NetCDFDataset(torch.utils.data.Dataset):
                 "batch_idx must be in the range" f" [0, {self.n_batches}), not {batch_idx}!"
             )
 
-        if self.cloud in ["gcp", "aws"]:
-            # download all data files
-            for data_source in self.data_sources_names:
-                data_source_and_filename = f"{data_source}/{get_netcdf_filename(batch_idx)}"
-                download_to_local(
-                    remote_filename=f"{self.src_path}/{data_source_and_filename}",
-                    local_filename=f"{self.tmp_path}/{data_source_and_filename}",
+        # get batches indexes
+        batch_indexes = [batch_idx]
+        if self.mix_two_batches:
+            second_batch_idx = np.random.randint(0, len(self) - 1)
+
+            # make sure second index is not the same as the first one
+            if second_batch_idx == batch_idx:
+                second_batch_idx = (second_batch_idx + 1) % len(self)
+
+            batch_indexes.append(second_batch_idx)
+
+        # download batches
+        batches = []
+        for batch_idx in batch_indexes:
+
+            if self.src_path != self.tmp_path:
+
+                batch = Batch.download_batch_and_load_batch(
+                    batch_idx=batch_idx,
+                    data_sources_names=self.data_sources_names,
+                    tmp_path=self.tmp_path,
+                    src_path=self.src_path,
                 )
+            else:
+                batch: Batch = Batch.load_netcdf(
+                    self.src_path,
+                    batch_idx=batch_idx,
+                    data_sources_names=self.data_sources_names,
+                )
+            batches.append(batch)
 
-            # download locations file
-            download_to_local(
-                remote_filename=f"{self.src_path}/"
-                f"{SPATIAL_AND_TEMPORAL_LOCATIONS_OF_EACH_EXAMPLE_FILENAME}",
-                local_filename=f"{self.tmp_path}/"
-                f"{SPATIAL_AND_TEMPORAL_LOCATIONS_OF_EACH_EXAMPLE_FILENAME}",
-            )
-
-            local_netcdf_folder = self.tmp_path
-        else:
-            local_netcdf_folder = self.src_path
-
-        batch: Batch = Batch.load_netcdf(
-            local_netcdf_folder, batch_idx=batch_idx, data_sources_names=self.data_sources_names
-        )
+        # join batches
+        batch = join_two_batches(batches=batches, data_sources_names=self.data_sources_names)
 
         if self.select_subset_data:
             batch = subselect_data(
@@ -201,7 +208,7 @@ class NetCDFDataset(torch.utils.data.Dataset):
             )
             raise e
 
-        if self.cloud != "local":
+        if self.src_path != self.tmp_path:
             # remove files in a folder, but not the folder itself
             delete_all_files_in_temp_path(self.tmp_path)
 
@@ -225,7 +232,6 @@ class SatFlowDataset(NetCDFDataset):
         src_path: str,
         tmp_path: str,
         configuration: Configuration,
-        cloud: str = "gcp",
         required_keys: Union[Tuple[str], List[str]] = None,
         history_minutes: Optional[int] = None,
         forecast_minutes: Optional[int] = None,
@@ -244,14 +250,12 @@ class SatFlowDataset(NetCDFDataset):
                 Google Cloud storage.
             tmp_path: The full path to the local temporary directory
                 (on a local filesystem).
-            cloud:
             required_keys: Tuple or list of keys required in the example for
                 it to be considered usable
             history_minutes: How many past minutes of data to use, if subsetting the batch
             forecast_minutes: How many future minutes of data to use, if reducing the amount
                 of forecast time
             configuration: configuration object
-            cloud: which cloud is used, can be "gcp", "aws" or "local".
             normalize: normalize the batch data
             add_position_encoding: Whether to add position encoding or not
             add_satellite_target: Whether to add future satellite imagery to the target or not
@@ -263,7 +267,6 @@ class SatFlowDataset(NetCDFDataset):
             src_path=src_path,
             tmp_path=tmp_path,
             configuration=configuration,
-            cloud=cloud,
             required_keys=required_keys,
             history_minutes=history_minutes,
             forecast_minutes=forecast_minutes,
